//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
// 

#include "Txc2.h"
#include "ExamplePacket_m.h"
#include <iostream>
using namespace std;

namespace tictoc {

Define_Module(Txc2);

void Txc2::initialize() {
    Txc::initialize();
    interval = &par("interval");
    sendPacket = new cMessage("sendPacket");
    seqNumber = 0;
}

void Txc2::handleMessage(cMessage *msg) {
    // just send back the message we received after some time
    if (msg == sendPacket) {
        // if this is the "sendPacket" self message, then send a new packet
        ExamplePacket *pkt = new ExamplePacket();
        pkt->setSender(getId());
        pkt->setSeqNumber(seqNumber++);
        send(pkt, "out");
    }
    else {
        ExamplePacket *rcvd = dynamic_cast<ExamplePacket *>(msg);
        if (rcvd != 0) {
            // this must be a packet from the other node
            cout << "Node " << getId() << " received packet #" <<
                    rcvd->getSeqNumber() << " from Node " <<
                    rcvd->getSender() << " at time " << simTime() << "\n";
            // free memory: very important!
            delete rcvd;
        }
        // we could also receive the "startup message" that is scheduled in Txc::initialize()
        // in both cases, schedule the sending of the next packet
        scheduleAt(simTime() + SimTime(interval->doubleValue()), sendPacket);
    }
}

void Txc2::finish() {
    cancelAndDelete(sendPacket);
    sendPacket = 0;
}

};
// namespace
